@@ -1,11 +1,14 @@
 DEFAULT_DATAKITCHEN_URL = 'https://cloud.datakitchen.io'
 KITCHEN = 'kitchen'
+KITCHEN_STAFF = 'kitchen-staff'
 RECIPE = 'recipe'
 VARIATION = 'variation'
 PARAMETERS = 'parameters'
 ORDER_ID = 'order_id'
 ORDER_RUN_ID = 'order_run_id'
 ORDER_RUN_STATUS = 'order_run_status'
+RECIPE_OVERRIDES = 'recipeoverrides'
+PARENT_KITCHEN = 'parent-kitchen'
 
 # API HTTP Request Methods
 API_GET = 'get'
